/*jshint esversion: 6 */
const responseTime = require('response-time');
const _ = require('lodash');
const urlSelector = require("url-selector");

const MASKED_ATTRIBUTES = [
	'clientKey',
	'clientSecret',
	'password',
	'ccNumber',
	'ccExpires',
	'ccCvc'
];

const maskSnsFields = (obj) => {
	obj = _.omit(obj, MASKED_ATTRIBUTES);

	return _.reduce(obj, (result, value, key) => {
		if (_.isArray(value)) {
			value = _.map(value, (subElement) => {
				if (_.isObject(subElement)) {
					return maskSnsFields(subElement);
				}

				return subElement;
			});
		} else if (_.isObject(value)) {
			value = maskSnsFields(value);
		}

		result[key] = value;
		return result;

	}, {});
};

const validate = (logger) => {
	if (!logger) throw new Error('[Express-Initializer] Provide a logger');
	if (!_.isFunction(logger.debug)) throw new Error('[Express-Initializer] Logger must have debug method');
	if (!_.isFunction(logger.error)) throw new Error('[Express-Initializer] Logger must have error method');
};

const setFormatedId = (req) => {
	req.formatedIp = req.ip;
	if (req.ip.includes('::ffff:')) {
		req.formatedIp = req.ip.substring(7);
	}
};

module.exports = (logger, config) => {
	let topicPrefix = config.publishers.topicPrefix ? config.publishers.topicPrefix : '';

	validate(logger);

	const Publisher = require('aws-worker');

	return (req, res, next) => {

		setFormatedId(req);

		return responseTime((req, res, time) => {
			setImmediate(() => {

				if (res.statusCode === 401) return;

				let getTopic = urlSelector.getTopic(config.publishers.whiteListTopic, req, topicPrefix, true);

				if (!getTopic) {
					return;
				}

				//OGM: parametreyi true geciyorum ama cok da emin degilim.
				req.serviceInit.organizationId = req.serviceInit.getOrganizationId(true);

				/* <code_review_item author:'OGM'>
				 *     asagida user setliyoruz ama currentUserin olmadigi senaryolar olabilir. currentClient gibi
				 * </code_review_item>*/
				let snsMessage = {
					message: {
						request: {
							path: req.route.path,
							params: maskSnsFields(req.params),
							body: maskSnsFields(req.body),
							query: maskSnsFields(req.query),
							method: req.method,
							requestId: req.serviceInit.requestId
						},
						response: {
							status: res.statusCode,
							responseTime: time,
							locals: res.locals
						},
						user: req.currentUser,
						serviceInit: req.serviceInit
					},
					topic: getTopic
				};

				Publisher.getService('SNS', config.publishers.config['AWS-SNS'])
					.send(snsMessage)
					.then(() => {
						logger.debug("OK: Sns Publish");
					})
					.catch((err) => {
						logger.error("Error: Sns Publish", {
							message: snsMessage,
							error: err
						});
					});
			});
		})(req, res, next);
	};
};